/*
 * cpu.c
 */

#include <stdio.h>
#include <stdlib.h>

#include "cpu.h"
#include "instruction.h"

/*
 *
 */

CPU *init_cpu ( CPU *cpup, int start ) {

  cpup -> pc = start;
  cpup -> acc = 0;
  cpup -> flag = 0;			/* all clear */

  return cpup;
}

/*
 *	a++ -> 値は、今の a の値で、その値を取り出したあと、a を 1 増やす
 */

void execute ( CPU *cpup, int memory[], int debug ) {
  int ic;
  int addr;

  while ( ( ic = memory[cpup -> pc++] ) != CPU_HALT ) {

	if ( debug == 1 ) {
		printf ( "%x:%d  acc=%d, flag=%d\n",
			 cpup -> pc - 1,
			 ic,
			 cpup -> acc,
			 cpup -> flag );
	}

	switch ( ic ) {
	case CPU_NOP:
	  break;
	case CPU_HALT:
	  /* 本当はここにこないが.. */
	  return;
	  break;
	case CPU_OUT:
	  putchar ( cpup -> acc );
	  break;
	case CPU_IN:
	  cpup -> acc = getchar();
	  break;
	case CPU_JUMP:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> pc = addr;
	  break;
	case CPU_JZ:
	  addr = memory[ cpup -> pc++ ];
	  if ( get_zero(cpup) ) {
		cpup -> pc = addr;
	  }
	  break;
	case CPU_JN:
	  addr = memory[ cpup -> pc++ ];
	  if ( get_negative(cpup) ) {
		cpup -> pc = addr;
	  }
	  break;
	case CPU_LDI:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc = addr;
	  break;
	case CPU_LDM:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc = memory[addr];
	  break;
	case CPU_STM:
	  addr = memory[ cpup -> pc++ ];
	  memory[addr] = cpup -> acc;
	  break;
	case CPU_ADD:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc += memory[addr];
	  clear_zero(cpup);
	  clear_negative(cpup);
	  if ( cpup -> acc == 0 ) {
		set_zero(cpup);
	  } else if ( cpup -> acc < 0 ) {
		set_negative(cpup);
	  }
	  break;
	case CPU_SUB:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc -= memory[addr];
	  clear_zero(cpup);
	  clear_negative(cpup);
	  if ( cpup -> acc == 0 ) {
		set_zero(cpup);
	  } else if ( cpup -> acc < 0 ) {
		set_negative(cpup);
	  }
	  break;
	default:
	  fprintf ( stderr, "Illigal Instruction(%d) at %d\n", ic, cpup -> pc - 1 );
	  exit ( -1 );
	  break;
	}
  }

  return;
}

/*
 *
 */

